<?php declare(strict_types=1);

namespace App\Http\Controllers\Private;

use App\Dto\QuerySettingsDto;
use App\Http\Requests\Private\Users\IndexRequest;
use App\Http\Requests\Private\Users\StoreUpdateRequest;
use App\Http\Requests\Private\Users\UpdatePasswordRequest;
use App\Services\Private\UserService;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\View\View;

class UsersController extends Controller
{
    public function __construct(
        private readonly UserService $userService
    ) { }

    public function index(IndexRequest $request): View
    {
        $user = $request->user();
        $data = $request->getDto();
        $querySettingsDto = new QuerySettingsDto(
            limit: 20,
            page: $data->getPage(),
            queryWith: []
        );

        $result = $this->userService->index($data->getUserBuilderDto(), $querySettingsDto, $user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('private/users/index', $result->getData());
    }

    public function create(Request $request): View
    {
        $user = $request->user();
        $result = $this->userService->create($user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('private/users/create', $result->getData());
    }

    public function edit(int $id, Request $request): View
    {
        $user = $request->user();
        $result = $this->userService->edit($id, $user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('private/users/edit', $result->getData());
    }

    public function store(StoreUpdateRequest $request): RedirectResponse
    {
        $data = $request->getDto();
        $user = $request->user();
        $result = $this->userService->store($data, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('users.edit', $result->getModel())->withSuccess($result->getMessage());
    }

    public function update(int $id, StoreUpdateRequest $request): RedirectResponse
    {
        $data = $request->getDto();
        $user = $request->user();
        $result = $this->userService->update($id, $data, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('users.edit', $result->getModel())->withSuccess($result->getMessage());
    }

    public function updatePassword(int $id, UpdatePasswordRequest $request): RedirectResponse
    {
        $data = $request->getDto();
        $user = $request->user();
        $result = $this->userService->updatePassword($id, $data, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('users.edit', $result->getModel())->withSuccess($result->getMessage());
    }

    public function destroy(int $id, Request $request): RedirectResponse
    {
        $user = $request->user();
        $result = $this->userService->destroy($id, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('users.index')->withSuccess($result->getMessage());
    }
}
